import numpy as np
import ot

from problem import Parameters



def cost_matrix(tau, tau_pred):
    """
    parameters
    ----------
    tau: double[r]
        ground truth delay times vector
    tau_pred: double[s]
        estimated delay times vector
    """
    r = tau.shape[0]
    s = tau_pred.shape[0]
    return tau_pred.reshape(s, 1) @ np.ones((1, r)) - np.ones((s, 1)) @ tau.reshape(1, r)


def optimal_transport_matrix(a, a_pred, D):
    """
    parameters
    ----------
    a: double[r]
        ground truth amplitudes
    a_pred: double[s]
        estimated amplitudes
    D: double[s, r]
        distance matrix
    """
    return ot.emd(a_pred, a, D)


def wasserstein1(theta, theta_pred):
    """
    parameters
    ----------
    theta: Parameters
        grand truth values
    theta_pred: Parameters
        estimated values
    """
    Dk = cost_matrix(theta.tau, theta_pred.tau)
    Gamma = optimal_transport_matrix(theta.a, theta_pred.a, np.abs(Dk))
    return np.sum(Gamma * np.abs(Dk))

